function [mesh] = spheroid_mesh(d, phi)
%SPHEROID_MESH  Generates a Cartesian mesh of d-dimensional scenarios
%                  based on the given ellipsoid.  This function does
%                  not assume that the ellipsoid is centered at the origin.
%
% Syntax:      [mesh] = spheroid_mesh(d, phi)
%
% Inputs:
%              d - the number of dimensions for the unit spheroid
%              phi - the scalar fineness of the mesh
%
% Outputs:
%              mesh - a d x N array with each column a scenario
%
% Example: 
%              % The basic Cartesian mesh
%              mesh = spheroid_mesh(d, phi);
%              mesh_size = length(mesh(1, :));
%              % Transform (to an ellipsoid) and recenter the shocks:
%              mesh = tfm_str\mesh;
%              mesh = tfm_rot\mesh;
%              mesh = mesh + mu * ones(1, mesh_size);
%
% Other m-files required: none
%
% Subfunctions: 
%              [total_pts, corner_pts, edge_pts, face_pts] = ...
%                 calc_mesh_size(phi, d)
%              [cmesh, shk_curs] = ...
%                 fill_corners(cmesh, shock, shk_curs, dim_curs)
%              [corn_mesh] = make_corners(d, normalize)
%              [edge_mesh] = make_edges(d, phi, normalize)
%              [face_mesh] = make_faces(d, phi, normalize)
%              [facemesh2D] = fill_adj_2Dface(d, phi)
%              [beta] = fill_adj_2Dface_beta(p, phi, z_one, z_phi)
%
% MAT-files required: none
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EITHER EXPRESS, IMPLIED, STATUTORY OR OTHERWISE INCLUDING BUT NOT LIMITED TO
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, FREE OF
% DEFECTS AND NONINFRINGEMENT. 
% 
% UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY OF LIABILITY, WHETHER TORT
% (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE SHALL THE AUTHORS OR THEIR 
% EMPLOYERS BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR 
% CONSEQUENTIAL DAMAGES OF ANY KIND, INCLUDING, WITHOUT LIMITATION, DAMAGES 
% FOR LOST PROFITS, LOSS OF GOODWILL, WORK STOPPPAGE, COMPUTER FAILURE OR 
% MALFUNCTION, LOSS OF USE, LOSS OF DATA, OR ANY AND ALL OTHER COMMERCIAL 
% DAMAGES AND LOSSES WHETHER OR NOT ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, 
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
% THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH 
% YOU. IF THE SOFTWARE PROVES DEFECTIVE IN ANY RESPECT, YOU ASSUME THE COST OF 
% ANY NECESSARY SERVICING, REPAIR OR CORRECTION  
% THIS SOFTWARE IS RELEASED INTO THE PUBLIC DOMAIN, OCT 2012.

% Author: Mark D. Flood
% Research and Analysis Center, Office of Financial Research
% Oct 2012; Last revision: 22-Nov-2010

%------------- BEGIN CODE --------------
    global face_template;

    %% Choose whether to normalize points from the cube onto the sphere
    normalize = true;
    % Memoize the raw 2D grid of points
    face_template = fill_adj_2Dface(d, phi);
     
    %% How many shocks will we get
    [mesh_size, corn_pts, edge_pts, face_pts] = calc_mesh_size(phi, d);
    fprintf('d=%.0f, phi=%.0f, mesh_size=%.0f, corn_pts=%.0f, edge_pts=%.0f, face_pts=%.0f\n', ... 
      d, phi, mesh_size, corn_pts, edge_pts, face_pts);

    %% The basic Cartesian mesh
    % Start with a mesh filled with 999s.  Why 999s?  Because 999s will
    % stand out from true mesh points, which are within the unit hypercube.
    mesh = ones(d, mesh_size)*999;
    
    %% Insert the corner points for the first corn_pts columns
    corn_mesh = make_corners(d, normalize);
    mesh(:, 1:corn_pts) = corn_mesh;

    %% Insert the non-corner edge points for the next edge_pts columns
    edge_mesh = make_edges(d, phi, normalize);
    edge_pts = length(edge_mesh(1,:));
    cursor_min = corn_pts+1;
    cursor_max = corn_pts+edge_pts;
    mesh(:, cursor_min:cursor_max) = edge_mesh;

    %% Insert the non-edge face points for the next face_pts columns
    face_mesh = make_faces(d, phi, normalize);
    face_pts = length(face_mesh(1,:));
    cursor_min = cursor_max+1;
    cursor_max = cursor_max+face_pts;
    mesh(:, cursor_min:cursor_max) = face_mesh;

end

function [total_pts, corner_pts, edge_pts, face_pts] = ...
  calc_mesh_size(phi, d)
    % Calculates the number of points in a mesh of fineness phi,
    % covering a hypercube in d dimensions
    %    corner_pts: count of points extreme (+/- 1) in all dimensions
    %    edge_pts:   count of points extreme in all but one dimension
    %    face_pts:   count of points extreme in all but two dimensions
    %    total_pts:  sum of:  corner_pts + edge_pts + face_pts
    corner_pts = 2^d;
    edges = d*2^(d-1);
    edge_pts = edges*(phi-2);
    faces = d*(d-1)*2^(d-3);
    face_pts = faces*(phi-2)*(phi-2);
    total_pts = corner_pts + edge_pts + face_pts;
end

function [cmesh, shk_curs] = fill_corners(cmesh, shock, shk_curs, dim_curs)
    % Systematically fills a given mesh array (cmesh) with d-dimensional 
    % points representing every corner of a d-dimensional hypercube.
    % The function fills the successive dimensions of eacy point
    % via depth-first recursion across all d dimensions.
    %    cmesh:    the mesh to be filled with corner points
    %    shock:    the current shock vector being filled
    %    shk_curs: index in cmesh of the shock currently being filled
    %    dim_curs: index in the current shock of the dimension being filled
    % Returns:
    %    cmesh:    d x 2^d array of corner points being filled
    %    shk_curs: last point in cmesh that was filled
    d = length(shock);
    for i=1:-2:-1;
        shock(dim_curs) = i;
        if (dim_curs<d)
            % Recurse through all d dimensions:
            [cmesh, shk_curs] = ...
              fill_corners(cmesh, shock, shk_curs, dim_curs+1);
        else
            shk_curs = shk_curs + 1;
            cmesh(:,shk_curs) = shock';
        end
    end
end

function [corn_mesh] = make_corners(d, normalize)
    % Fills a mesh (corn_mesh) with d-dimensional points representing all
    % corners of a d-dimensional cube encompassing a d-dimensional unit
    % spheroid.  
    %    d:         the number of dimensions to cover
    %    normalize: whether to scale the corner points onto the sphere
    % Returns:
    %    corn_mesh: a d x 2^d array of corner points
    corn_mesh = fill_corners(ones(d, 2^d)*111, zeros(d, 1), 0, 1);
    if (normalize)
        % Scale every point to unit length
        scal = norm(corn_mesh(:, 1));
        corn_mesh = corn_mesh / scal;
    end
end

function [edge_mesh] = make_edges(d, phi, normalize)
    % Fills a mesh (edge_mesh) with d-dimensional points representing all
    % non-corner edge points of a d-dimensional cube encompassing a 
    % d-dimensional unit spheroid.  
    %    d:         the number of dimensions to cover
    %    phi:       fineness of the mesh along the edge (i.e., the total
    %               number of points, *including* the corners)
    %    normalize: whether to scale the edge points onto the sphere
    % Returns:
    %    edge_mesh: a d x d*2^(d-1)*(phi-2) array of edge points
    global face_template;
    edge_pts = d*2^(d-1)*(phi-2);
    edge_mesh = ones(d, edge_pts)*222;
    corner_mesh = make_corners(d-1, false);
    corner_rows = length(corner_mesh(1,:));
    corner_cols = length(corner_mesh(:,1));
    patch = zeros(d, corner_rows);
    cursor = 1;
    for active_edge = 1:d
        for p=2:phi-1
            edge_vals = face_template(1, p, d)*ones(1, corner_rows);
            cmax = cursor+corner_rows-1;
            patch(1:active_edge-1, 1:corner_rows) = ...
              corner_mesh(1:active_edge-1, 1:corner_rows);
            patch(active_edge, 1:corner_rows) = ...
              edge_vals(1, :);
            patch(active_edge+1:d, 1:corner_rows) = ...
              corner_mesh(active_edge:corner_cols, 1:corner_rows);
            if (normalize) 
                patch = patch / norm(patch(:, 1));
            end
            edge_mesh(1:d, cursor:cmax) = patch;
            cursor = cursor + corner_rows;
        end
    end
end

function [face_mesh] = make_faces(d, phi, normalize)
    % Fills a mesh (face_mesh) with d-dimensional points representing all
    % non-edge face points of a d-dimensional cube encompassing a 
    % d-dimensional unit spheroid.  
    %    d:         the number of dimensions to cover
    %    phi:       fineness of the mesh along each dimension of the 2D
    %               face
    %    normalize: whether to scale the face points onto the sphere
    % Returns:
    %    face_mesh: a d x d*(d-1)*2^(d-3)*(phi-2)^2 array of face points
    global face_template;
    face_pts = d*(d-1)*2^(d-3)*(phi-2)*(phi-2);
    face_mesh = ones(d, face_pts)*333;
    corner_mesh = make_corners(d-2, false);
    corner_rows = length(corner_mesh(1,:));
    corner_cols = length(corner_mesh(:,1));
    patch = zeros(d, corner_rows);
    cursor = 1;
    for active_edge1 = 1:d
        for active_edge2 = active_edge1+1:d
            for p1=2:phi-1
                face_vals = face_template(p1, p1, d)*ones(2, corner_rows);
                for p2=2:phi-1
                    face_vals(2:2, 1:corner_rows) = ...
                      face_template(p2, p2, d)*ones(1, corner_rows);
                    cmax = cursor+corner_rows-1;
                    patch(1:active_edge1-1, 1:corner_rows) = ...
                      corner_mesh(1:active_edge1-1, 1:corner_rows);
                    patch(active_edge1, 1:corner_rows) = ...
                      face_vals(1, :);
                    patch(active_edge1+1:active_edge2-1, 1:corner_rows) = ...
                      corner_mesh(active_edge1:active_edge2-2, 1:corner_rows);
                    patch(active_edge2, 1:corner_rows) = ...
                      face_vals(2, :);
                    patch(active_edge2+1:d, 1:corner_rows) = ...
                      corner_mesh(active_edge2-1:corner_cols, 1:corner_rows);
                    if (normalize) 
                        patch = patch / norm(patch(:, 1));
                    end
                    face_mesh(1:d, cursor:cmax) = patch;
                    cursor = cursor + corner_rows;
                end
            end
        end
    end
end

function [facemesh2D] = fill_adj_2Dface(d, phi)
    % Creates a phi x phi grid (i.e., the mesh on a single
    % two-dimensional face of a larger hypercube) of d-dimensional
    % points, where the regularity of the grid has been adjusted to avoid 
    % clustering in the corners.  Thus, facemesh2D is a phi x phi x d
    % array of points.  The points (each facemesh2D(i,j,:)) are identically
    % equal to one in the first d-2 dimensions, so that the mesh varies
    % only in the final two dimensions.
    facemesh2D = zeros(phi, phi, d);

    % perim holds an example of the points along one of the four edges
    % of the square face. There are phi points along each edge.
    % Since the four edges are symmetrical, we only need to calculate 
    % one edge example.
    perim = ones(phi, d);
    perim(1, d) = -1;
    z_one = perim(1,:);
    z_phi = perim(phi,:);
    for p=2:phi-1;
        beta = fill_adj_2Dface_beta(p, phi, z_one, z_phi);
        perim(p,:) = beta*z_one + (1-beta)*z_phi;
    end;
        
    % Interpolating from the edge points in perim to get the interior 
    % points of each row/column.
    for i=1:phi;
        z_one = ones(1,d);
        z_phi = ones(1,d);
        z_one(1,d) = -1;
        z_one(1,d-1) = perim(i,d);
        z_phi(1,d-1) = perim(i,d);
        for j=1:phi;
            beta = fill_adj_2Dface_beta(j, phi, z_one, z_phi);
            facemesh2D(i, j, :) = beta*z_one + (1-beta)*z_phi;
            %msg = 'i=%.0f, j=%.0f,\n b=%.5f,\n b1 =[%.5f, %.5f, %.5f],\n bp =[%.5f, %.5f, %.5f],\n sum=[%.5f, %.5f, %.5f]';
            %disp(sprintf(msg, i, j, beta, betafoo1, betafoop, facemesh2D(i, j, :)));
        end;
    end;
    % Now set the rows to achieve symmetry with the columns
    for i=1:phi;
        for j=1:phi;
            facemesh2D(j, i, d-1) = facemesh2D(i, j, d);
        end;
    end;
end

function [beta] = fill_adj_2Dface_beta(p, phi, z_one, z_phi)
    % Calculates the factor, beta in [0, 1], that interpolates the pth
    % equidistant point between the two endpoints, z_one and z_phi, for
    % and adjusted 2D mesh of fineness phi in d dimensions.
    n_one = norm(z_one);
    n_phi = norm(z_phi);
    zTone = z_one*z_one';
    zTphi = z_phi*z_phi';
    zToxp = z_one*z_phi';
    theta = acos(zToxp/(n_one*n_phi))/(phi-1);
    g_one = n_one*cos((p-1)*theta);
    g_phi = n_phi*cos((phi-p)*theta);
    beta = zTphi*g_one-zToxp*g_phi;
    beta = beta/((zTphi-zToxp)*g_one+(zTone-zToxp)*g_phi);
    if (abs(beta)<.00000000000001) 
        beta = 0;
    end
end


