function [s] = vandercorput(k, b)
%VANDERCORPUT Generates a base-b Van der Corput sequence with k elements
%
% Syntax:      [s] = vandercorput(k, b)
%
% Inputs:
%              k - number of elements in the sequence
%              b - base for integer expansions used in the sequence
%
% Outputs:
%              s - k-dimensional array containing the sequence
%
% Example: 
%              vdcs = vandercorput(15, 2);
%              assertEquals('vdcs(1) s/b 0.0000', vdcs(1), 0.0000);
%              assertEquals('vdcs(2) s/b 0.5000', vdcs(1), 0.5000);
%              assertEquals('vdcs(3) s/b 0.2500', vdcs(2), 0.2500);
%
% Other m-files required: 
%              baseb_expansion(i, b)

% Subfunctions: none
% MAT-files required: none
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EITHER EXPRESS, IMPLIED, STATUTORY OR OTHERWISE INCLUDING BUT NOT LIMITED TO
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, FREE OF
% DEFECTS AND NONINFRINGEMENT. 
% 
% UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY OF LIABILITY, WHETHER TORT
% (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE SHALL THE AUTHORS OR THEIR 
% EMPLOYERS BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR 
% CONSEQUENTIAL DAMAGES OF ANY KIND, INCLUDING, WITHOUT LIMITATION, DAMAGES 
% FOR LOST PROFITS, LOSS OF GOODWILL, WORK STOPPPAGE, COMPUTER FAILURE OR 
% MALFUNCTION, LOSS OF USE, LOSS OF DATA, OR ANY AND ALL OTHER COMMERCIAL 
% DAMAGES AND LOSSES WHETHER OR NOT ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, 
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
% THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH 
% YOU. IF THE SOFTWARE PROVES DEFECTIVE IN ANY RESPECT, YOU ASSUME THE COST OF 
% ANY NECESSARY SERVICING, REPAIR OR CORRECTION  
% THIS SOFTWARE IS RELEASED INTO THE PUBLIC DOMAIN, OCT 2012.

% Author: Mark D. Flood
% Research and Analysis Center, Office of Financial Research
% Oct 2012; Last revision: 22-Nov-2010

%------------- BEGIN CODE --------------
    if (k ~= floor(k) || (k < 0))
       error('k=%.7f is NOT a non-negative integer.', k)
    end
    if (b ~= floor(b) || (b < 2))
       error('b=%.7f is NOT a positive integer greater than 1.', b)
    end
    s = zeros(k+1,1);
    for i = 1:k
        xi = 0;
        q = 1/b;
        a = baseb_expansion(i, b);
        m = length(a);
        for j = 1:m
            xi = xi + q*a(m-j+1);
            q = q/b;
        end
        s(i+1) = xi;
    end    
end

